@@ -41,6 +41,9 @@
 
 #include <math.h>
 
+gpointer ags_soundcard_util_copy(gpointer ptr);
+void ags_soundcard_util_free(gpointer ptr);
+
 /**
  * SECTION:ags_soundcard_util
  * @short_description: soundcard util
@@ -51,6 +54,41 @@
  * Soundcard utility functions.
  */
 
+GType
+ags_soundcard_util_get_type(void)
+{
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if(g_once_init_enter (&g_define_type_id__volatile)){
+    GType ags_type_soundcard_util = 0;
+
+    ags_type_soundcard_util =
+      g_boxed_type_register_static("AgsSoundcardUtil",
+				   (GBoxedCopyFunc) ags_soundcard_util_copy,
+				   (GBoxedFreeFunc) ags_soundcard_util_free);
+
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_soundcard_util);
+  }
+
+  return g_define_type_id__volatile;
+}
+
+gpointer
+ags_soundcard_util_copy(gpointer ptr)
+{
+  gpointer retval;
+
+  retval = g_memdup(ptr, sizeof(AgsSoundcardUtil));
+ 
+  return(retval);
+}
+
+void
+ags_soundcard_util_free(gpointer ptr)
+{
+  g_free(ptr);
+}
+
 /**
  * ags_soundcard_util_get_obj_mutex:
  * @soundcard: the #GObject sub-type implementing #AgsSoundcard
