@@ -172,6 +172,9 @@ ags_fx_vst3_audio_init(AgsFxVst3Audio *fx_vst3_audio)
   fx_vst3_audio->input_port_count = 0;
   fx_vst3_audio->input_port = NULL;
 
+  fx_vst3_audio->program_port_index = -1;
+  fx_vst3_audio->program_param_id = -1;
+  
   /* scope data */
   for(i = 0; i < AGS_SOUND_SCOPE_LAST; i++){
     if(i == AGS_SOUND_SCOPE_PLAYBACK ||
@@ -275,7 +278,9 @@ ags_fx_vst3_audio_notify_effect_callback(GObject *gobject,
 					 GParamSpec *pspec,
 					 gpointer user_data)
 {
-  //TODO:JK: implement me
+  AGS_FX_VST3_AUDIO(gobject)->vst3_plugin = ags_vst3_manager_find_vst3_plugin(ags_vst3_manager_get_instance(),
+									      AGS_RECALL(gobject)->filename,
+									      AGS_RECALL(gobject)->effect);
 }
 
 void
@@ -523,6 +528,13 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 
   GList *start_recall_channel, *recall_channel;
 
+#if defined(HAVE_GLIB_2_68)
+  GStrvBuilder *strv_builder;
+#endif
+  
+  gchar **parameter_name;
+    
+  guint n_params;
   guint input_pads;
   guint output_port_count, input_port_count;
   guint buffer_size;
@@ -530,6 +542,8 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
   guint i, j, k;
   gboolean is_live_instrument;
 
+  GValue *value;
+
   GRecMutex *recall_mutex;
 
   /* get recall mutex */
@@ -547,6 +561,29 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
     return;
   }
 
+#if defined(HAVE_GLIB_2_68)
+  strv_builder = g_strv_builder_new();
+
+  g_strv_builder_add(strv_builder,
+		     "buffer-size");
+  g_strv_builder_add(strv_builder,
+		     "samplerate");
+  g_strv_builder_add(strv_builder,
+		     "iedit-controller");
+  g_strv_builder_add(strv_builder,
+		     "iaudio-processor");
+  
+  parameter_name = g_strv_builder_end(strv_builder);
+#else
+  parameter_name = (gchar **) g_malloc(5 * sizeof(gchar *));
+
+  parameter_name[0] = g_strdup("buffer-size");
+  parameter_name[1] = g_strdup("samplerate");
+  parameter_name[2] = g_strdup("iedit-controller");
+  parameter_name[3] = g_strdup("iaudio-processor");
+  parameter_name[4] = NULL;
+#endif
+
   start_input = NULL;
 
   input_pads = 0;
@@ -578,6 +615,29 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 
   g_rec_mutex_unlock(recall_mutex);
 
+  n_params = 4;
+
+  value = g_new0(GValue,
+		 4);
+
+  g_value_init(value,
+	       G_TYPE_UINT);
+    
+  g_value_set_uint(value,
+		   buffer_size);
+    
+  g_value_init(value + 1,
+	       G_TYPE_UINT);
+
+  g_value_set_uint(value + 1,
+		   samplerate);
+    
+  g_value_init(value + 2,
+	       G_TYPE_POINTER);
+
+  g_value_init(value + 3,
+	       G_TYPE_POINTER);
+
   /* allocate channel data */
   is_live_instrument = ags_fx_vst3_audio_test_flags(fx_vst3_audio, AGS_FX_VST3_AUDIO_LIVE_INSTRUMENT);
     
@@ -620,6 +680,13 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 	  
 	    guint nth;
 
+	    channel_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
+									       &n_params,
+									       &parameter_name, &value);
+	      
+	    channel_data->iedit_controller = g_value_get_pointer(value + 2);
+	    channel_data->iaudio_processor = g_value_get_pointer(value + 3);
+
 	    ags_vst_process_data_set_num_samples(channel_data->process_data,
 						 buffer_size);
 	  
@@ -650,6 +717,13 @@ ags_fx_vst3_audio_set_audio_channels_callback(AgsAudio *audio,
 
 	      input_data = channel_data->input_data[k];
 	    
+	      input_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
+									       &n_params,
+									       &parameter_name, &value);
+
+	      input_data->iedit_controller = g_value_get_pointer(value + 2);
+	      input_data->iaudio_processor = g_value_get_pointer(value + 3);
+
 	      ags_vst_process_data_set_num_samples(input_data->process_data,
 						   buffer_size);
 	      
@@ -1353,7 +1427,7 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
 		input_data->icomponent = ags_base_plugin_instantiate_with_params((AgsBasePlugin *) vst3_plugin,
 										 &n_params,
 										 &parameter_name, &value);
-	      
+
 		input_data->iedit_controller = g_value_get_pointer(value + 2);
 		input_data->iaudio_processor = g_value_get_pointer(value + 3);
 	      }
@@ -1374,6 +1448,14 @@ ags_fx_vst3_audio_load_plugin(AgsFxVst3Audio *fx_vst3_audio)
   g_free(effect);
 }
 
+/**
+ * ags_fx_vst3_audio_load_port:
+ * @fx_vst3_audio: the #AgsFxVst3Audio
+ * 
+ * Load port of @fx_vst3_audio.
+ * 
+ * Since: 3.10.5
+ */
 void
 ags_fx_vst3_audio_load_port(AgsFxVst3Audio *fx_vst3_audio)
 {
@@ -1392,6 +1474,7 @@ ags_fx_vst3_audio_load_port(AgsFxVst3Audio *fx_vst3_audio)
   
   guint input_pads;
   guint audio_channels;
+  guint key;
   guint output_port_count, input_port_count;
   guint control_port_count;
   guint buffer_size;
@@ -1465,8 +1548,10 @@ ags_fx_vst3_audio_load_port(AgsFxVst3Audio *fx_vst3_audio)
   output_port = NULL;
   input_port = NULL;
   
-  output_port_count = 0;
-  input_port_count = 0;
+  output_port_count = ags_vst3_plugin_get_audio_output_port_count(vst3_plugin,
+								  0);
+  input_port_count = ags_vst3_plugin_get_audio_input_port_count(vst3_plugin,
+								0);
   
   control_port_count = 0;
 
@@ -1661,6 +1746,12 @@ ags_fx_vst3_audio_load_port(AgsFxVst3Audio *fx_vst3_audio)
 
 	if(is_live_instrument){	  
 	  guint nth;
+
+	  ags_vst_process_data_set_num_inputs(channel_data->process_data,
+					      input_port_count);
+
+	  ags_vst_process_data_set_num_outputs(channel_data->process_data,
+					       output_port_count);
 	  
 	  if(channel_data->output == NULL &&
 	     output_port_count > 0 &&
@@ -1673,6 +1764,33 @@ ags_fx_vst3_audio_load_port(AgsFxVst3Audio *fx_vst3_audio)
 	     buffer_size > 0){
 	    channel_data->input = (float *) g_malloc(input_port_count * buffer_size * sizeof(float));
 	  }
+	}else{
+	  for(key = 0; key < AGS_SEQUENCER_MAX_MIDI_KEYS; key++){
+	    AgsFxVst3AudioInputData *input_data;
+	  
+	    AgsVstParamID param_id;
+
+	    input_data = channel_data->input_data[key];
+
+	    ags_vst_process_data_set_num_inputs(input_data->process_data,
+						input_port_count);
+
+	    ags_vst_process_data_set_num_outputs(input_data->process_data,
+						 output_port_count);
+	  
+	    if(input_data->output == NULL &&
+	       output_port_count > 0 &&
+	       buffer_size > 0){
+	      input_data->output = (float *) g_malloc(output_port_count * buffer_size * sizeof(float));
+	    }
+	  
+	    if(input_data->input == NULL &&
+	       input_port_count > 0 &&
+	       buffer_size > 0){
+	      input_data->input = (float *) g_malloc(input_port_count * buffer_size * sizeof(float));
+	    }
+
+	  }
 	}
       }
     }
@@ -1713,8 +1831,10 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
   guint port_index;
   gfloat param_value;
   gboolean is_live_instrument;
+
+  GRecMutex *fx_vst3_audio_mutex;
   
-//  g_message("write %s", port->specifier);
+  fx_vst3_audio_mutex = AGS_RECALL_GET_OBJ_MUTEX(fx_vst3_audio);
 
   audio = NULL;
   plugin_port = NULL;
@@ -1749,6 +1869,8 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
 
   info = ags_vst_parameter_info_alloc();      
   
+  g_rec_mutex_lock(fx_vst3_audio_mutex);
+  
   for(sound_scope = 0; sound_scope < AGS_SOUND_SCOPE_LAST; sound_scope++){
     AgsFxVst3AudioScopeData *audio_scope_data;
 
@@ -1801,6 +1923,8 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
     }
   }
 
+  g_rec_mutex_unlock(fx_vst3_audio_mutex);
+
   ags_vst_parameter_info_free(info);
   
   if(audio != NULL){
@@ -1812,6 +1936,153 @@ ags_fx_vst3_audio_safe_write_callback(AgsPort *port, GValue *value,
   }
 }
 
+/**
+ * ags_fx_vst3_audio_load_plugin:
+ * @fx_vst3_audio: the #AgsFxVst3Audio
+ * @port_index: the port index
+ * @program_list_id: the program list id
+ * @program_index: the program index
+ * 
+ * Change program of @fx_vst3_audio.
+ * 
+ * Since: 3.10.10
+ */
+void
+ags_fx_vst3_audio_change_program(AgsFxVst3Audio *fx_vst3_audio,
+				 guint port_index,
+				 guint program_list_id,
+				 guint program_index)
+{
+  AgsAudio *audio;
+  
+  AgsVstIProgramListData *iprogram_list_data;
+  AgsVstParameterInfo *info;
+  
+  guint audio_channels;
+  gint sound_scope;
+  gfloat param_value;
+  gboolean is_live_instrument;
+  AgsVstTResult val;
+
+  GRecMutex *fx_vst3_audio_mutex;
+  
+  fx_vst3_audio_mutex = AGS_RECALL_GET_OBJ_MUTEX(fx_vst3_audio);
+
+  if(!AGS_IS_FX_VST3_AUDIO(fx_vst3_audio)){
+    return;
+  }
+
+  if(fx_vst3_audio->vst3_plugin == NULL ||
+     !ags_base_plugin_test_flags((AgsBasePlugin *) fx_vst3_audio->vst3_plugin, AGS_BASE_PLUGIN_IS_INSTRUMENT)){
+    return;
+  }
+  
+  audio = NULL;
+
+  audio_channels = 0;
+
+  port_index = 0;
+  
+  g_object_get(fx_vst3_audio,
+	       "audio", &audio,
+	       NULL);
+
+  if(audio != NULL){
+    g_object_get(audio,
+		 "audio-channels", &audio_channels,
+		 NULL);
+  }
+  
+  is_live_instrument = ags_fx_vst3_audio_test_flags(fx_vst3_audio, AGS_FX_VST3_AUDIO_LIVE_INSTRUMENT);
+
+  param_value = 1.0;
+
+  info = ags_vst_parameter_info_alloc();      
+  
+  g_rec_mutex_lock(fx_vst3_audio_mutex);
+  
+  for(sound_scope = 0; sound_scope < AGS_SOUND_SCOPE_LAST; sound_scope++){
+    AgsFxVst3AudioScopeData *audio_scope_data;
+
+    guint channel;
+    
+    audio_scope_data = fx_vst3_audio->scope_data[sound_scope];
+
+    if(sound_scope == AGS_SOUND_SCOPE_PLAYBACK ||
+       sound_scope == AGS_SOUND_SCOPE_NOTATION ||
+       sound_scope == AGS_SOUND_SCOPE_MIDI){
+      for(channel = 0; channel < audio_channels; channel++){
+	AgsFxVst3AudioChannelData *channel_data;
+
+	guint key;
+	
+	channel_data = audio_scope_data->channel_data[channel];
+
+	if(is_live_instrument){
+	  AgsVstParamID param_id;
+
+	  if(channel_data->iedit_controller != NULL){
+	    iprogram_list_data = NULL;
+	    
+	    val = ags_vst_funknown_query_interface(channel_data->iedit_controller,
+						   ags_vst_iprogram_list_data_get_iid(), &iprogram_list_data);
+
+	    if(iprogram_list_data != NULL){
+	      AgsVstIBStream *data;
+
+	      data = NULL;
+	      
+	      ags_vst_iprogram_list_data_get_program_data(iprogram_list_data,
+							  program_list_id, program_index,
+							  data);
+	    
+	      ags_vst_iedit_controller_set_state(channel_data->iedit_controller,
+						 data);
+	    }
+	  }
+	}else{	
+	  for(key = 0; key < AGS_SEQUENCER_MAX_MIDI_KEYS; key++){
+	    AgsFxVst3AudioInputData *input_data;
+	  
+	    AgsVstParamID param_id;
+
+	    input_data = channel_data->input_data[key];
+
+	    if(input_data->iedit_controller != NULL){
+	      iprogram_list_data = NULL;
+	    
+	      val = ags_vst_funknown_query_interface(input_data->iedit_controller,
+						     ags_vst_iprogram_list_data_get_iid(), &iprogram_list_data);
+
+
+	      if(iprogram_list_data != NULL){
+		AgsVstIBStream *data;
+
+		data = NULL;
+	      
+		ags_vst_iprogram_list_data_get_program_data(iprogram_list_data,
+							    program_list_id, program_index,
+							    data);
+	    
+		ags_vst_iedit_controller_set_state(input_data->iedit_controller,
+						   data);
+	      }
+	    }
+	  }
+	}
+      }
+    }
+  }
+  
+  g_rec_mutex_unlock(fx_vst3_audio_mutex);
+
+  ags_vst_parameter_info_free(info);
+  
+  if(audio != NULL){
+    g_object_unref(audio);
+  }
+}
+
 /**
  * ags_fx_vst3_audio_new:
  * @audio: the #AgsAudio
