@@ -291,14 +291,8 @@ ags_fx_vst3_audio_signal_real_run_inter(AgsRecall *recall)
 						  buffer_size, copy_mode_in);
     }
 
-    ags_base_plugin_activate(vst3_plugin,
-			     input_data->icomponent);
-    
     ags_vst_iaudio_processor_process(input_data->iaudio_processor,
 				     input_data->process_data);  
-
-    ags_base_plugin_deactivate(vst3_plugin,
-			       input_data->icomponent);
     
     if(input_data->output != NULL &&
        fx_vst3_channel->output_port_count >= 1 &&
@@ -470,31 +464,27 @@ ags_fx_vst3_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_audio
       g_rec_mutex_lock(fx_vst3_audio_mutex);
       
       input_data->key_on += 1;
+      
+      if(is_live_instrument){
+	ags_vst_ievent_list_add_event(channel_data->input_event,
+				      ags_vst_note_on_event_alloc(0,
+								  midi_note,
+								  0.0,
+								  1.0,
+								  -1,
+								  -1));
+      }else{
+	ags_vst_ievent_list_add_event(input_data->input_event,
+				      ags_vst_note_on_event_alloc(0,
+								  midi_note,
+								  0.0,
+								  1.0,
+								  -1,
+								  -1));
+      }            
 
       g_rec_mutex_unlock(fx_vst3_audio_mutex);
     }
-
-    g_rec_mutex_lock(fx_vst3_audio_mutex);
-    
-    if(is_live_instrument){
-      ags_vst_ievent_list_add_event(channel_data->input_event,
-				    ags_vst_note_on_event_alloc(0,
-								midi_note,
-								0.0,
-								1.0,
-								-1,
-								-1));
-    }else{
-      ags_vst_ievent_list_add_event(input_data->input_event,
-				    ags_vst_note_on_event_alloc(0,
-								midi_note,
-								0.0,
-								1.0,
-								-1,
-								-1));
-    }      
-
-    g_rec_mutex_unlock(fx_vst3_audio_mutex);
     
     if(is_live_instrument){
       g_rec_mutex_lock(fx_vst3_audio_mutex);
@@ -505,15 +495,8 @@ ags_fx_vst3_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_audio
       }
 
       if(input_data->iaudio_processor != NULL){
-	ags_base_plugin_activate(vst3_plugin,
-				 input_data->icomponent);
-	
 	ags_vst_iaudio_processor_process(input_data->iaudio_processor,
 					 input_data->process_data);  
-
-
-	ags_base_plugin_deactivate(vst3_plugin,
-				   input_data->icomponent);
       }
 
       g_rec_mutex_lock(source_stream_mutex);
@@ -539,14 +522,8 @@ ags_fx_vst3_audio_signal_stream_feed(AgsFxNotationAudioSignal *fx_notation_audio
       }
 
       if(input_data->iaudio_processor != NULL){
-	ags_base_plugin_activate(vst3_plugin,
-				 input_data->icomponent);
-	
 	ags_vst_iaudio_processor_process(input_data->iaudio_processor,
 					 input_data->process_data);  
-
-	ags_base_plugin_deactivate(vst3_plugin,
-				   input_data->icomponent);
       }
       
       g_rec_mutex_lock(source_stream_mutex);
