@@ -161,6 +161,10 @@ ags_vst3_plugin_init(AgsVst3Plugin *vst3_plugin)
 							 NULL);
   }
 
+  vst3_plugin->program = g_hash_table_new_full(g_direct_hash, g_str_equal,
+					       NULL,
+					       NULL);
+
   vst3_plugin->get_plugin_factory = NULL;
 
   vst3_plugin->host_context = NULL;
@@ -275,6 +279,7 @@ ags_vst3_plugin_instantiate_with_params(AgsBasePlugin *base_plugin,
   g_rec_mutex_unlock(base_plugin_mutex);
 
   retval = NULL;
+  icomponent = NULL;
 
   if(GetPluginFactory != NULL){    
     AgsVstPClassInfo *info;
@@ -296,8 +301,6 @@ ags_vst3_plugin_instantiate_with_params(AgsBasePlugin *base_plugin,
       }
 
       /* icomponent */
-      icomponent = NULL;
-
       val = ags_vst_iplugin_factory_create_instance(iplugin_factory,
 						    ags_vst_pclass_info_get_cid(info),
 						    ags_vst_icomponent_get_iid(),
@@ -389,7 +392,7 @@ ags_vst3_plugin_instantiate_with_params(AgsBasePlugin *base_plugin,
     }  
     
     /* return value */
-    if((position = ags_strv_index(parameter_name[0], "iaudio-processor")) >= 0){
+    if((position = ags_strv_index(parameter_name[0], "iaudio-processor")) >= 0){      
       g_value_set_pointer(&(value[0][position]),
 			  iaudio_processor);
     }    
@@ -472,6 +475,7 @@ ags_vst3_plugin_load_plugin(AgsBasePlugin *base_plugin)
 
   GError *error;
 
+  gboolean (*InitDll)();
   AgsVstIPluginFactory* (*GetPluginFactory)();
   
   GRecMutex *base_plugin_mutex;
@@ -505,6 +509,22 @@ ags_vst3_plugin_load_plugin(AgsBasePlugin *base_plugin)
 
   success = FALSE;
 
+#ifdef AGS_W32API
+  InitDll = GetProcAddress(base_plugin->plugin_so,
+			   "InitDll");
+    
+  success = (AGS_VST3_PLUGIN(base_plugin)->get_plugin_factory != NULL) ? TRUE: FALSE;
+#else
+  InitDll = dlsym(base_plugin->plugin_so,
+		  "InitDll");
+  
+  success = (dlerror() == NULL) ? TRUE: FALSE;
+#endif
+
+  if(InitDll){
+    InitDll();
+  }
+    
 #ifdef AGS_W32API
   GetPluginFactory =
     AGS_VST3_PLUGIN(base_plugin)->get_plugin_factory = GetProcAddress(base_plugin->plugin_so,
@@ -632,6 +652,28 @@ ags_vst3_plugin_load_plugin(AgsBasePlugin *base_plugin)
 	gint32 step_count;
 	AgsVstParamValue default_normalized_value;
       
+	info = ags_vst_parameter_info_alloc();
+      
+	ags_vst_iedit_controller_get_parameter_info(AGS_VST3_PLUGIN(base_plugin)->iedit_controller,
+						    i, info);
+
+	flags = ags_vst_parameter_info_get_flags(info);
+
+	if((AGS_VST_KIS_PROGRAM_CHANGE & (flags)) != 0){
+	  gchar *program;
+	  
+	  program = g_utf16_to_utf8(ags_vst_parameter_info_get_title(info),
+				    128,
+				    NULL,
+				    NULL,
+				    &error);
+	    
+	  g_hash_table_insert(AGS_VST3_PLUGIN(base_plugin)->program,
+			      program, GUINT_TO_POINTER(i));
+
+	  continue;
+	}
+	
 	current_plugin_port = ags_plugin_port_new();
 	g_object_ref(current_plugin_port);
 
@@ -644,13 +686,6 @@ ags_vst3_plugin_load_plugin(AgsBasePlugin *base_plugin)
 		     G_TYPE_FLOAT);
 	g_value_init(current_plugin_port->upper_value,
 		     G_TYPE_FLOAT);
-      
-	info = ags_vst_parameter_info_alloc();
-      
-	ags_vst_iedit_controller_get_parameter_info(AGS_VST3_PLUGIN(base_plugin)->iedit_controller,
-						    i, info);
-
-	flags = ags_vst_parameter_info_get_flags(info);
 
 	step_count = ags_vst_parameter_info_get_step_count(info);
 
@@ -744,10 +779,6 @@ ags_vst3_plugin_load_plugin(AgsBasePlugin *base_plugin)
 	  //TODO:JK: implement me
 	}
       
-	if((AGS_VST_KIS_PROGRAM_CHANGE & (flags)) != 0){
-	  //TODO:JK: implement me
-	}
-      
 	if((AGS_VST_KIS_BYPASS & (flags)) != 0){
 	  current_plugin_port->flags |= AGS_PLUGIN_PORT_TOGGLED;
 	}      
@@ -860,6 +891,96 @@ ags_vst3_plugin_get_event_input_bus_count(AgsVst3Plugin *vst3_plugin)
   return(event_input_bus_count);
 }
 
+/**
+ * ags_vst3_plugin_get_audio_output_port_count:
+ * @vst3_plugin: the #AgsVst3Plugin
+ * @bus_index: the bus index
+ * 
+ * Get audio output port count.
+ * 
+ * Returns: the audio output port count
+ * 
+ * Since: 3.10.10
+ */
+guint
+ags_vst3_plugin_get_audio_output_port_count(AgsVst3Plugin *vst3_plugin,
+					    guint bus_index)
+{
+  guint audio_output_port_count;
+
+  audio_output_port_count = 0;
+  
+  if(vst3_plugin->icomponent != NULL){
+    AgsVstIAudioProcessor *iaudio_processor;
+    
+    AgsVstSpeakerArrangement arrangement;
+
+    guint i;
+    
+    iaudio_processor = NULL;
+  
+    ags_vst_funknown_query_interface(vst3_plugin->icomponent,
+				     ags_vst_iaudio_processor_get_iid(), &iaudio_processor);
+    
+    ags_vst_iaudio_processor_get_bus_arrangement(iaudio_processor,
+						 AGS_VST_KOUTPUT, bus_index,
+						 &arrangement);
+
+    for(i = 0; i < 8 * sizeof(AgsVstSpeakerArrangement); i++){
+      if((1 << i) & arrangement != 0){
+	audio_output_port_count++;
+      }
+    }
+  }
+
+  return(audio_output_port_count);
+}
+
+/**
+ * ags_vst3_plugin_get_audio_input_port_count:
+ * @vst3_plugin: the #AgsVst3Plugin
+ * @bus_index: the bus index
+ * 
+ * Get audio input port count.
+ * 
+ * Returns: the audio input port count
+ * 
+ * Since: 3.10.10
+ */
+guint
+ags_vst3_plugin_get_audio_input_port_count(AgsVst3Plugin *vst3_plugin,
+					   guint bus_index)
+{
+  guint audio_input_port_count;
+
+  audio_input_port_count = 0;
+  
+  if(vst3_plugin->icomponent != NULL){
+    AgsVstIAudioProcessor *iaudio_processor;
+    
+    AgsVstSpeakerArrangement arrangement;
+
+    guint i;
+    
+    iaudio_processor = NULL;
+  
+    ags_vst_funknown_query_interface(vst3_plugin->icomponent,
+				     ags_vst_iaudio_processor_get_iid(), &iaudio_processor);
+    
+    ags_vst_iaudio_processor_get_bus_arrangement(iaudio_processor,
+						 AGS_VST_KINPUT, bus_index,
+						 &arrangement);
+
+    for(i = 0; i < 8 * sizeof(AgsVstSpeakerArrangement); i++){
+      if((1 << i) & arrangement != 0){
+	audio_input_port_count++;
+      }
+    }
+  }
+
+  return(audio_input_port_count);
+}
+
 /**
  * ags_vst3_plugin_process_data_lookup:
  * @icomponent: the icomponent as key
